from multiprocessing.pool import Pool
import matplotlib.pyplot as plt
import numpy as np
import math
from mcmcFunc3Cat import *
from RNGen import *
from subprg import *

#
#       Yasuharu Okamoto, 2016.11
#

if __name__ == '__main__':

    X, n, meanSt, sgmSt, fout, nm_fout = InputData()

    NSimu = 1000
    
    params = {'jump' : 0,
              'n' : n,
              'X' : X,
              'NSimu' : NSimu,
              'initC1' : meanSt - sgmSt,
              'initC2' : meanSt + sgmSt,
              'initSgm' : sgmSt,
              'genSgmC1' : sgmSt/10.0,
              'genSgmC2' : sgmSt/10.0,
              'genSgmSgm' : sgmSt/10.0}
    #
    #               Adjust the parameters in MCMC
    #
    istep = 0
    while True:
        print("Step-{} started.".format(istep))
        
        C1, C2, Sgm, cnt_C1, cnt_C2, cnt_sgm = mcmcFunc( params )

        sum_C1 = 0.0
        sum_C2 = 0.0
        sum_sgm = 0.0
        for t in range(1, NSimu + 1):
            sum_C1 += C1[t]
            sum_C2 += C2[t]
            sum_sgm += Sgm[t]
        params['initC1'] = sum_C1 / NSimu
        params['initC2'] = sum_C2 / NSimu
        params['initSgm'] = sum_sgm / NSimu
        print("meanC1 = {}".format(params['initC1']))
        print('meanC2 = {}'.format(params['initC2']))
        print("meanSgm = {}".format(params['initSgm']))
                
        ck_cnt = 0
        r_C1 = cnt_C1 / NSimu
        print("r_C1 = {}".format(r_C1))
        v, ck_cnt = scale_sgm( r_C1, ck_cnt )
        params['genSgmC1'] = params['genSgmC1'] * v
        print("genSgmC1 = {}".format(params['genSgmC1']))
        r_C2 = cnt_C2 / NSimu
        print("r_C2 = {}".format(r_C2))
        v, ck_cnt = scale_sgm( r_C2, ck_cnt )
        params['genSgmC2'] = params['genSgmC2'] * v
        print("genSgmC2 = {}".format(params['genSgmC2']))
        r_sgm = cnt_sgm / NSimu;
        print("r_sgm = {}".format(r_sgm))
        v, ck_cnt = scale_sgm( r_sgm, ck_cnt )
        params['genSgmSgm'] = params['genSgmSgm'] * v
        print("genSgmSgm = {}".format(params['genSgmSgm']))
        print("ck_cnt = {}".format(ck_cnt))
        if ck_cnt == 0:
            break
        istep += 1

    #
    #           The main MCMCs
    #
    NSimu = 3000
    params['NSimu'] = NSimu
    n_mcmc = 4
    ary_params = []
    for i in range(0, n_mcmc):
        temp = params.copy()
        temp['jump'] = i + 1
        ary_params.append(temp)

    print("\nThe main MCMCs started.")
    pool = Pool()
    results = pool.map(mcmcFunc, ary_params)
    print("\nThe main MCMCs ended.")
    mrg_C1 = []
    mrg_C2 = []
    mrg_sgm = []
    for temp_C1, temp_C2, temp_Sgm, cnt_C1, cnt_C2, cnt_sgm in results:
        mrg_C1 += temp_C1[1:NSimu + 1]
        mrg_C2 += temp_C2[1:NSimu + 1]
        mrg_sgm += temp_Sgm[1:NSimu + 1]
    print("Length(mrg_C1) = {}".format(len(mrg_C1)))
    print('Length(mrg_C2) = {}'.format(len(mrg_C2)))
    print("Length(mrg_sgm) = {}".format(len(mrg_sgm)))
    mrg_C1.sort()
    mrg_C2.sort()
    mrg_sgm.sort()

    DispResults( NSimu, n_mcmc, X, n, mrg_C1, mrg_C2, mrg_sgm, fout )

    fout.close()
    print("\nOutput file {} was saved.".format(nm_fout))

