import math
import matplotlib.pyplot as plt
import numpy as np
from RNGen import *
from common3Cat import *


def InputData():
    
    nm_fin = input("Input data file = ")
    fin = open(nm_fin, 'rt')

    nm_fout = input('Output file name = ')
    fout = open(nm_fout, 'wt')
    fout.write("Input data file...{}\n".format(nm_fin))

    data = fin.readlines()
    fin.close()

    pos = 0
    while True:
        if len(data[pos]) > 0:
            if data[pos][0] == '/':
                break
        pos += 1

    ID = []
    X = []
    pos += 1
    while True:
        if data[pos][0] == '/':
            break
        temp = data[pos].split()
        print(temp)
        ID.append(temp[0])
        tempL = []
        tempL.append(float(temp[1]))
        tempL.append(int(temp[2]))
        X.append(tempL)
        pos += 1

    n = len(X)
    for i in range(0, n):
        fout.write("{0:>5}   {1}    {2}\n".format(ID[i], X[i][0], X[i][1]))

    sumSt = 0.0
    for i in range(0, n):
        sumSt += X[i][0]
    meanSt = sumSt / n
    ssumSt = 0.0
    for i in range(0, n):
        ssumSt += (X[i][0] - meanSt) ** 2.0
    varSt = ssumSt / n
    sgmSt = math.sqrt(varSt)

    return X, n, meanSt, sgmSt, fout, nm_fout



def DispResults( NSimu, n_mcmc, X, n, mrg_C1, mrg_C2, mrg_sgm, fout ):

    Q1C1 = mrg_C1[int(NSimu * n_mcmc * 0.25)]    
    MedC1 = mrg_C1[int(NSimu * n_mcmc / 2.0)]
    Q3C1 = mrg_C1[int(NSimu * n_mcmc * 0.75)]
    LC1 = mrg_C1[int(NSimu * n_mcmc * 0.025)]
    UC1 = mrg_C1[int(NSimu * n_mcmc * 0.975)]

    Q1C2 = mrg_C2[int(NSimu * n_mcmc * 0.25)]
    MedC2 = mrg_C2[int(NSimu * n_mcmc / 2.0)]
    Q3C2 = mrg_C2[int(NSimu * n_mcmc * 0.75)]
    LC2 = mrg_C2[int(NSimu * n_mcmc * 0.025)]
    UC2 = mrg_C2[int(NSimu * n_mcmc * 0.975)]

    Q1Sgm = mrg_sgm[int(NSimu * n_mcmc * 0.25)]
    MedSgm = mrg_sgm[int(NSimu * n_mcmc / 2.0)]
    Q3Sgm = mrg_sgm[int(NSimu * n_mcmc * 0.75)]
    LSgm = mrg_sgm[int(NSimu * n_mcmc * 0.025)]
    USgm = mrg_sgm[int(NSimu * n_mcmc * 0.975)]
    
    print("Med. of C1 = {}".format(MedC1))
    print('Med. of C2 = {}'.format(MedC2))
    print("Med. of Sgm = {}".format(MedSgm))

    fout.write("\nMed. of C1 = {0:.5}\n".format(MedC1))
    fout.write("Q1(C1) = {0:.5}    Q3(C1) = {1:.5}\n".format(Q1C1, Q3C1))
    fout.write("95% CI((C1) = [{0:.5}, {1:.5}]\n".format(LC1, UC1))

    fout.write("\nMed. of C2 = {0:.5}\n".format(MedC2))
    fout.write("Q1(C2) = {0:.5}    Q3(C2) = {1:.5}\n".format(Q1C2, Q3C2))
    fout.write("95% CI((C2) = [{0:.5}, {1:.5}]\n".format(LC2, UC2))

    fout.write("\nMed. of JND = {0:.5}\n".format(MedSgm * 0.67449))
    fout.write("Q1(JND) = {0:.5}    Q3(JND) = {1:.5}\n".format(Q1Sgm * 0.67449, Q3Sgm * 0.67449 ))
    fout.write("95% CI(JND) = [{0:.5}, {1:.5}]\n".format(LSgm * 0.67449, USgm * 0.67449))

    min_St = X[0][0]
    max_St = X[0][0]
    for i in range(1, n):
        if min_St > X[i][0]:
            min_St = X[i][0]
        if max_St < X[i][0]:
            max_St = X[i][0]
    min_x = min_St - 0.05 * (max_St - min_St)
    max_x = max_St + 0.05 * (max_St - min_St)
  
    plt.axis([min_x, max_x, 0.0, 1.0])

    phy = []
    psy1 = []
    psy2 = []
    v = min_x
    while v <= max_x:
        phy.append(v)
        psy1.append(cum_normalKC((v - MedC1) / MedSgm))
        psy2.append(cum_normalKC((v - MedC2) / MedSgm))
        v += 0.01 * (max_x - min_x)

    rn = RNp2to191()
    p_st = []
    p_res = []
    for i in range(0, n):
        p_st.append(X[i][0])
        if X[i][1] == 1:
            p_res.append(0.1 * rn.uni())
        elif X[i][1] == 2:
            p_res.append(0.45 + 0.1 * rn.uni())
        else:
            p_res.append(1.0 - 0.1 * rn.uni())
                    
            
    plt.title("Psychometric functions\nC1 = {0:.5},  C2 = {1:.5},  JND = {2:.5}".format(
                MedC1, MedC2, MedSgm * 0.67449))
    plt.plot(phy, psy1, 'b--', linewidth = 5)
    plt.plot(phy, psy2, 'b-.', linewidth = 5)
    plt.plot(p_st, p_res, 'go')
    plt.legend(['PM1', 'PM2', 'Data'], loc = 2)
    plt.show()

    min_x = mrg_C1[0]
    if min_x > mrg_C2[0]:
        min_x = mrg_C2[0]
    max_x = mrg_C1[n_mcmc * NSimu - 1]
    if max_x < mrg_C2[n_mcmc * NSimu - 1]:
        max_x = mrg_C2[n_mcmc * NSimu - 1]
    n_intvl = 20
    v_intvl = []
    for i in range(0, n_intvl):
        v_intvl.append(min_x + (i + 0.5) * (max_x - min_x) / n_intvl) 
    cnt1 = [0] * n_intvl
    cnt2 = [0] * n_intvl
    w_x = (max_x - min_x) / n_intvl
    for i in range(0, n_mcmc * NSimu):
        cnt1[intvl_pos(mrg_C1[i], min_x, w_x, n_intvl)] += 1
        cnt2[intvl_pos(mrg_C2[i], min_x, w_x, n_intvl)] += 1

    plt.title('C1(Med.) = {0:.5}     C2(Med.) = {1:.5}'.format(MedC1, MedC2))
    plt.plot(v_intvl, cnt1, 'b^', v_intvl, cnt2, 'bs')
    plt.plot(v_intvl, cnt1, 'b',  v_intvl, cnt2, 'b', linewidth = 3)
    plt.show()
        

    plt.title("JND(Med. = {0:.5})".format(MedSgm * 0.67449))
    JNDs = []
    for v in mrg_sgm:
        JNDs.append(v * 0.67449)
    plt.hist(JNDs, bins = 20)
    plt.show()
    
